import os
from typing import List

from core.experiment.config import ExperimentConfiguration, TreeConfiguration


class ComparePaths(object):
    """Class to get the path to different files for an experiment of DQN and SDT actions."""

    @staticmethod
    def directory(experiment_configuration: ExperimentConfiguration):
        path = f"{experiment_configuration.save_directory}/Comparison/"
        return path

    @staticmethod
    def log_actions_path(experiment_configuration: ExperimentConfiguration):
        directory = ComparePaths.directory(experiment_configuration)
        path = f"{directory}/actions.csv"
        return path

    @staticmethod
    def log_overview_actions_path(experiment_configuration: ExperimentConfiguration):
        directory = ComparePaths.directory(experiment_configuration)
        path = f"{directory}/actions_overview.csv"
        return path

    @staticmethod
    def log_tree_config_directory(experiment_configuration: ExperimentConfiguration,
                                  tree_configuration: TreeConfiguration):
        directory = ComparePaths.directory(experiment_configuration)
        path = f"{directory}/{tree_configuration.name}/"
        return path

    @staticmethod
    def log_specific_path(experiment_configuration: ExperimentConfiguration,
                          tree_configuration: TreeConfiguration,
                          sdt_callback_name):
        directory = ComparePaths.log_tree_config_directory(experiment_configuration, tree_configuration)
        path = f"{directory}/{sdt_callback_name}_actions.csv"
        return path

    @staticmethod
    def vis_directory(experiment_configuration: ExperimentConfiguration):
        directory = ComparePaths.directory(experiment_configuration)
        path = f"{directory}/vis/"
        return path

    @staticmethod
    def vis_per_tree_directory(experiment_configuration: ExperimentConfiguration,
                               tree_configuration: TreeConfiguration):
        directory = ComparePaths.vis_directory(experiment_configuration)
        path = f"{directory}/per_tree/{tree_configuration.name}/"
        return path

    @staticmethod
    def vis_per_callback_directory(experiment_configuration: ExperimentConfiguration,
                                   sdt_callback_name):
        directory = ComparePaths.vis_directory(experiment_configuration)
        path = f"{directory}/per_callback/{sdt_callback_name}/"
        return path

    @staticmethod
    def vis_per_all(experiment_configuration: ExperimentConfiguration):
        directory = ComparePaths.vis_directory(experiment_configuration)
        path = f"{directory}/all/"
        return path

    @staticmethod
    def vis_post_concurrent_directory(experiment_configuration1: ExperimentConfiguration,
                                      experiment_configuration2: ExperimentConfiguration):
        paths = [experiment_configuration1.save_directory, experiment_configuration2.save_directory]
        directory = os.path.commonpath(paths)
        path = f"{directory}/vis/{experiment_configuration1.seed}/"
        return path

    @staticmethod
    def vis_post_concurrent_tree(experiment_configuration1: ExperimentConfiguration,
                                 experiment_configuration2: ExperimentConfiguration,
                                 tree_configuration: TreeConfiguration,
                                 sdt_callback_name):
        directory = ComparePaths.vis_post_concurrent_directory(experiment_configuration1, experiment_configuration2)
        path = f"{directory}/{tree_configuration.name}/{sdt_callback_name}"
        return path

    @staticmethod
    def vis_full_comparison_directory(experiment_configurations: List[ExperimentConfiguration], no_seed=False):
        paths = list(map(lambda e: e.save_directory, experiment_configurations))
        directory = os.path.commonpath(paths)
        path = f"{directory}/vis/{'' if no_seed else experiment_configurations[0].seed}/"
        return path

    @staticmethod
    def vis_full_comparison_tree(experiment_configurations: List[ExperimentConfiguration],
                                 tree_configuration: TreeConfiguration, sdt_callback_name):
        directory = ComparePaths.vis_full_comparison_directory(experiment_configurations)
        path = f"{directory}/{tree_configuration.name}/{sdt_callback_name}"
        return path

    @staticmethod
    def vis_full_comparison_dqn(experiment_configurations: List[ExperimentConfiguration], no_seed=False):
        directory = ComparePaths.vis_full_comparison_directory(experiment_configurations, no_seed)
        path = f"{directory}/DQN/"
        return path

    @staticmethod
    def vis_avg_comparison_directory(experiment_configuration: ExperimentConfiguration):
        path = f"{experiment_configuration.save_directory.parent}/vis/"
        return path

    @staticmethod
    def vis_avg_dqn_comparison_directory(experiment_configuration: ExperimentConfiguration, training_dqn,
                                         use_best=False):
        directory = ComparePaths.vis_avg_comparison_directory(experiment_configuration)
        path = f"{directory}/DQN/{'best ' if use_best else ''}{'train' if training_dqn else 'test'}"
        return path

    @staticmethod
    def vis_all_dqn_directory(experiment_configuration: ExperimentConfiguration):
        directory = ComparePaths.vis_avg_comparison_directory(experiment_configuration)
        path = f"{directory}/DQN/"
        return path

    @staticmethod
    def vis_avg_sdt_directory(experiment_configuration: ExperimentConfiguration):
        directory = ComparePaths.vis_avg_comparison_directory(experiment_configuration)
        path = f"{directory}/SDT/"
        return path

    @staticmethod
    def vis_avg_all_sdt_directory(experiment_configuration: ExperimentConfiguration):
        directory = ComparePaths.vis_avg_comparison_directory(experiment_configuration)
        path = f"{directory}/SDT/all"
        return path

    @staticmethod
    def vis_avg_tree_config_directory(experiment_configuration: ExperimentConfiguration,
                                      tree_configuration: TreeConfiguration):
        directory = ComparePaths.vis_avg_sdt_directory(experiment_configuration)
        path = f"{directory}/SDT/{tree_configuration.name}/"
        return path

    @staticmethod
    def vis_avg_callback_directory(experiment_configuration: ExperimentConfiguration, sdt_callback_name):
        directory = ComparePaths.vis_avg_sdt_directory(experiment_configuration)
        path = f"{directory}/SDT/{sdt_callback_name}/"
        return path

    @staticmethod
    def vis_avg_specific_sdt_directory(experiment_configuration: ExperimentConfiguration,
                                       tree_configuration: TreeConfiguration, sdt_callback_name):
        directory = ComparePaths.vis_avg_tree_config_directory(experiment_configuration, tree_configuration)
        path = f"{directory}/{sdt_callback_name}/"
        return path
