from core.experiment.config import ExperimentConfiguration


class DQNPaths(object):
    """Class to get the path to different files for a DQN agent."""

    @staticmethod
    def directory(experiment_configuration: ExperimentConfiguration):
        path = f"{experiment_configuration.save_directory}/DQN/"
        return path

    @staticmethod
    def save_path(experiment_configuration: ExperimentConfiguration, step_save=None):
        directory = DQNPaths.directory(experiment_configuration)
        if step_save is None:
            step_save = experiment_configuration.steps
        path = f"{directory}/dqn_step-{step_save}.pt"
        return path

    @staticmethod
    def save_best_path(experiment_configuration: ExperimentConfiguration):
        directory = DQNPaths.directory(experiment_configuration)
        path = f"{directory}/dqn_best.pt"
        return path

    @staticmethod
    def save_best_log(experiment_configuration: ExperimentConfiguration, training_dqn):
        directory = DQNPaths.log_directory(experiment_configuration, training_dqn, best_save=True)
        path = f"{directory}/dqn_best.csv"
        return path

    @staticmethod
    def log_directory(experiment_configuration: ExperimentConfiguration, training_dqn, best_save=False):
        directory = DQNPaths.directory(experiment_configuration)
        if training_dqn:
            path = f"{directory}/train_log/"
        else:
            if best_save:
                path = f"{directory}/best_test_log/"
            else:
                path = f"{directory}/test_log/"
        return path

    @staticmethod
    def step_logs_path(experiment_configuration: ExperimentConfiguration, training_dqn, episode_interval=None,
                       best_save=False):
        directory = DQNPaths.log_directory(experiment_configuration, training_dqn, best_save)
        if episode_interval is None:
            episode_interval = "end"
        path = f"{directory}/step_data_{episode_interval}.csv"
        return path

    @staticmethod
    def episode_logs_path(experiment_configuration: ExperimentConfiguration, training_dqn, episode_interval=None,
                          best_save=False):
        directory = DQNPaths.log_directory(experiment_configuration, training_dqn, best_save)
        if episode_interval is None:
            episode_interval = "end"
        path = f"{directory}/episode_data_{episode_interval}.csv"
        return path

    @staticmethod
    def memory_logs_path(experiment_configuration: ExperimentConfiguration, training_dqn, episode_interval=None,
                         best_save=False):
        directory = DQNPaths.log_directory(experiment_configuration, training_dqn, best_save)
        if episode_interval is None:
            episode_interval = "end"
        path = f"{directory}/memory_data_{episode_interval}.csv"
        return path

    @staticmethod
    def vis_log_directory(experiment_configuration: ExperimentConfiguration, training_dqn, best_save):
        directory = DQNPaths.log_directory(experiment_configuration, training_dqn, best_save)
        path = f"{directory}/vis/"
        return path
