import os

from core.experiment.config import TreeConfiguration, ExperimentConfiguration


class SDTPaths(object):
    """Class to get the path to different files for a Soft Decision Tree."""

    @staticmethod
    def all_sdt_directory(experiment_configuration: ExperimentConfiguration):
        path = f"{experiment_configuration.save_directory}/SDT/"
        return path

    @staticmethod
    def directory(experiment_configuration: ExperimentConfiguration,
                  tree_configuration: TreeConfiguration,
                  sdt_callback_name):
        directory = SDTPaths.all_sdt_directory(experiment_configuration)
        path = f"{directory}/{tree_configuration.name}/{sdt_callback_name}/"
        return path

    @staticmethod
    def save_path(experiment_configuration: ExperimentConfiguration,
                  tree_configuration: TreeConfiguration,
                  sdt_callback_name, epoch_save):
        directory = SDTPaths.directory(experiment_configuration, tree_configuration, sdt_callback_name)
        path = f"{directory}/epoch-{epoch_save}.pt"
        return path

    @staticmethod
    def last_save_path(experiment_configuration: ExperimentConfiguration,
                       tree_configuration: TreeConfiguration,
                       sdt_callback_name):
        directory = SDTPaths.directory(experiment_configuration, tree_configuration, sdt_callback_name)
        last_epoch = 0
        for file in os.listdir(directory):
            if file.endswith(".pt"):
                epoch = int(file.replace(".pt", "").replace("epoch-", ""))
                last_epoch = max(last_epoch, epoch)
        path = f"{directory}/epoch-{last_epoch}.pt"
        return path

    @staticmethod
    def log_directory(experiment_configuration: ExperimentConfiguration,
                      tree_configuration: TreeConfiguration,
                      sdt_callback_name):
        directory = SDTPaths.directory(experiment_configuration, tree_configuration, sdt_callback_name)
        path = f"{directory}/log/"
        return path

    @staticmethod
    def log_losses_path(experiment_configuration: ExperimentConfiguration,
                        tree_configuration: TreeConfiguration,
                        sdt_callback_name, epoch_interval=None):
        directory = SDTPaths.log_directory(experiment_configuration, tree_configuration, sdt_callback_name)
        if epoch_interval is None:
            epoch_interval = "end"
        path = f"{directory}/losses_{epoch_interval}.csv"
        return path

    @staticmethod
    def log_actions_path(experiment_configuration: ExperimentConfiguration,
                         tree_configuration: TreeConfiguration,
                         sdt_callback_name, epoch_interval=None):
        directory = SDTPaths.log_directory(experiment_configuration, tree_configuration, sdt_callback_name)
        if epoch_interval is None:
            epoch_interval = "end"
        path = f"{directory}/actions_{epoch_interval}.csv"
        return path

    @staticmethod
    def log_post_actions_path(experiment_configuration: ExperimentConfiguration,
                              tree_configuration: TreeConfiguration,
                              sdt_callback_name, action):
        directory = SDTPaths.log_directory(experiment_configuration, tree_configuration, sdt_callback_name)
        path = f"{directory}/post_actions_data_{action}.csv"
        return path

    @staticmethod
    def log_weights_directory(experiment_configuration: ExperimentConfiguration,
                              tree_configuration: TreeConfiguration,
                              sdt_callback_name):
        directory = SDTPaths.log_directory(experiment_configuration, tree_configuration, sdt_callback_name)
        path = f"{directory}/nodes_parameters/"
        return path

    @staticmethod
    def log_inner_path(experiment_configuration: ExperimentConfiguration,
                       tree_configuration: TreeConfiguration,
                       sdt_callback_name, node_idx, epoch_interval=None):
        directory = SDTPaths.log_weights_directory(experiment_configuration, tree_configuration, sdt_callback_name)
        if epoch_interval is None:
            epoch_interval = "end"
        path = f"{directory}/inner_node_{node_idx}_{epoch_interval}.csv"
        return path

    @staticmethod
    def log_leaf_path(experiment_configuration: ExperimentConfiguration,
                      tree_configuration: TreeConfiguration,
                      sdt_callback_name, node_idx, epoch_interval=None):
        directory = SDTPaths.log_weights_directory(experiment_configuration, tree_configuration, sdt_callback_name)
        if epoch_interval is None:
            epoch_interval = "end"
        path = f"{directory}/leaf_node_{node_idx}_{epoch_interval}.csv"
        return path

    @staticmethod
    def vis_directory(experiment_configuration: ExperimentConfiguration):
        directory = SDTPaths.all_sdt_directory(experiment_configuration)
        path = f"{directory}/vis/"
        return path

    @staticmethod
    def vis_specific_directory(experiment_configuration: ExperimentConfiguration,
                               tree_configuration: TreeConfiguration,
                               sdt_callback_name):
        directory = SDTPaths.directory(experiment_configuration, tree_configuration, sdt_callback_name)
        path = f"{directory}/vis/"
        return path

    @staticmethod
    def vis_per_tree_directory(experiment_configuration: ExperimentConfiguration,
                               tree_configuration: TreeConfiguration):
        directory = SDTPaths.all_sdt_directory(experiment_configuration)
        path = f"{directory}/{tree_configuration.name}/vis/"
        return path

    @staticmethod
    def vis_per_callback_directory(experiment_configuration: ExperimentConfiguration,
                                   sdt_callback_name):
        directory = SDTPaths.vis_directory(experiment_configuration)
        path = f"{directory}/{sdt_callback_name}/"
        return path

    @staticmethod
    def vis_nodes_directory(experiment_configuration: ExperimentConfiguration,
                            tree_configuration: TreeConfiguration,
                            sdt_callback_name):
        directory = SDTPaths.vis_specific_directory(experiment_configuration, tree_configuration, sdt_callback_name)
        path = f"{directory}/nodes_parameters/"
        return path

    @staticmethod
    def vis_tree_nodes_directory(experiment_configuration: ExperimentConfiguration,
                                 tree_configuration: TreeConfiguration,
                                 sdt_callback_name):
        directory = SDTPaths.vis_specific_directory(experiment_configuration, tree_configuration, sdt_callback_name)
        path = f"{directory}/tree_vis/"
        return path
