import torch.nn as nn
from torch.optim import Adam

from core.experiment.config import *
from experiments.templates import get_tree_configurations


###########################
# ExperimentConfiguration #
###########################
env_name = "CartPole-v1"
steps = 200000
no_cuda = True


######################
# AgentConfiguration #
######################
warm_up_steps = 10000
agent_configuration = AgentConfiguration(
    # Policies
    warm_up_steps=warm_up_steps,
    policy=None,
    test_policy=None,
    # Experience replay
    memory_size=10000,
    batch_size=32,
    gamma=0.99,
    # Double DQN
    enable_double_dqn=True,
    target_update=1000,
    target_tau=1,
    # Saving checkpoints
    save_interval=int(steps / 10)
)


######################
# ModelConfiguration #
######################
# Method to create the model
def create_model(input_shape, output_shape):
    # The number of units in the layers
    units_layer1 = 16
    units_layer2 = 16
    # Create a sequential model
    model = nn.Sequential(
        # Layer 1
        nn.Linear(input_shape[0], units_layer1),
        nn.BatchNorm1d(units_layer1),
        nn.ReLU(),
        # Layer 2
        nn.Linear(units_layer1, units_layer2),
        nn.BatchNorm1d(units_layer2),
        nn.ReLU(),
        # Output layer
        nn.Linear(units_layer2, output_shape)
    )
    # Return the model
    return model


model_configuration = ModelConfiguration(
    create_function=create_model,
    optimizer=Adam,
    learning_rate=0.001,
    loss=nn.MSELoss
)

#####################
# TreeConfiguration #
#####################
# Static configuration values
lmbda = 0.1
sdt_batch_size = 32
epochs = 10
save_epoch_interval = epochs * 10
log_interval = epochs

# The collections of depths to generate configurations for
depths = [1, 2, 3, 4]
# The list of tree configurations
tree_configurations = get_tree_configurations(depths, lmbda, sdt_batch_size)


#####################
# Episode Callbacks #
#####################
# The number of episodes between each SDT training process,
#   along with the number of episodes to use for 1 training process.
episode_templates = [
    # (episode_interval, episodes)
]
_eps_intervals = [1, 5, 10, 20, 25, 50, 100]
for idx, episode_interval in enumerate(_eps_intervals):
    for eps in _eps_intervals[:idx + 1]:
        temp = (episode_interval, eps)
        episode_templates.append(temp)
episodes = 100
