from core.experiment.environment_description import get_env_description
from core.experiment.experiment import Experiment
from core.models.dqn.policy import DecayingEpsilonGreedyPolicy, GreedyPolicy

from experiments.cart_pole import *
from experiments.templates import get_episode_callbacks
from visualization.dqn.dqn_visualization import DQNVisualizer
from visualization.policy.policy_visualization import PolicyVisualizer


###########################
# ExperimentConfiguration #
###########################
experiment_configuration = ExperimentConfiguration(
    env_name=env_name,
    seed=9,
    steps=steps,
    no_cuda=no_cuda,
    save_directory=f"../../../../Data/Experiments/CartPole/epsilon_greedy/concurrent/Distributed_SDT_Replay/"
)
# Whether or not SDTs are trained together with the DQN
train_agent_only = False
# Whether or not the DQN agent is training at the same time as the SDT
training_agent = True


######################
# AgentConfiguration #
######################
# Set the experiment specific policies
agent_configuration.policy = DecayingEpsilonGreedyPolicy(epsilon_max=1.0, epsilon_min=0.1, warm_up_steps=warm_up_steps,
                                                         steps=int((steps - warm_up_steps) * 0.5))
agent_configuration.test_policy = GreedyPolicy()

agent_configuration.sdt_replay = True
agent_configuration.memory_size = 2000  # => 1000 per leaf for 2 leafs
agent_configuration.random_sampling = False
agent_configuration.prioritised_sampling = False


#####################
# Episode Callbacks #
#####################
# The collections of depths to generate configurations for
depths = [1]
# The list of tree configurations
tree_configurations = get_tree_configurations(depths, lmbda, sdt_batch_size)

episode_templates = [
    # (episode_interval, episodes)
    (100, 10)
]
# Generate the episode callbacks
episode_callbacks = get_episode_callbacks(episode_templates, training_agent, agent_configuration.warm_up_steps,
                                          epochs, save_epoch_interval, log_interval)

##############
# Experiment #
##############
experiment = Experiment(experiment_configuration, agent_configuration, model_configuration,
                        tree_configurations, episode_callbacks, sdt_threads=8)
env_description = get_env_description(experiment_configuration.env_name)


if __name__ == '__main__':
    gym.logger.set_level(gym.logger.DEBUG)
    torch.set_num_threads(1)  # Any other value than single core creates overhead

    # # Train DQN agent
    experiment.train(train_agent_only=train_agent_only, verbose=1)
    # # Plot DQN results
    dqn_visualizer = DQNVisualizer()
    num_leafs = 2 ** depths[0]
    dqn_visualizer.plot_all(experiment_configuration, training_dqn=True, num_leafs=num_leafs, colour_idx=3)
    # Test and plot best save
    experiment.test_agent(episodes, log_results=True, visualize=False, use_best=True)
    dqn_visualizer.plot_all(experiment_configuration, training_dqn=False, use_best=True, num_leafs=num_leafs)

    # Test the final agent and train the SDTs
    experiment.test_agent(episodes, log_results=True, visualize=False)
    dqn_visualizer.plot_all(experiment_configuration, training_dqn=False, num_leafs=num_leafs)

    # Compare all agents
    # dqn_visualizer = DQNVisualizer()
    # dqn_visualizer.compare_same_config(experiment_configuration, seeds=range(30))
    # dqn_visualizer.plot_seeds_all(experiment_configuration, seeds=range(30))

    # policy_visualizer = PolicyVisualizer()
    # policy_visualizer.plot_avg_comparisons(experiment_configuration, tree_configurations, episode_callbacks,
    #                                        training_agent, range(30), plot_if_one=True)
