from core.experiment.environment_description import get_env_description
from core.experiment.experiment import Experiment

from experiments.lunar_lander import *
from experiments.templates import get_episode_callbacks
from visualization.dqn.dqn_visualization import DQNVisualizer
from visualization.policy.policy_visualization import PolicyVisualizer
from visualization.sdt.sdt_visualization import SDTVisualizer
from visualization.sdt.tree_visualization import TreeVisualizer


###########################
# ExperimentConfiguration #
###########################
experiment_configuration = ExperimentConfiguration(
    env_name=env_name,
    seed=0,
    steps=steps,
    no_cuda=no_cuda,
    save_directory=f"../../../../Data/Experiments/LunarLander/epsilon_greedy/concurrent/Default/"
)
# Whether or not SDTs are trained together with the DQN
train_agent_only = False
# Whether or not the DQN agent is training at the same time as the SDT
training_agent = True


#####################
# Episode Callbacks #
#####################
# The collections of depths to generate configurations for
depths = [5, 8]
# The list of tree configurations
tree_configurations = get_tree_configurations(depths, lmbda, sdt_batch_size)

episode_templates = [
    # (episode_interval, episodes)
]
_eps_intervals = [25, 50, 100]
for idx, episode_interval in enumerate(_eps_intervals):
    for eps in _eps_intervals[:idx + 1]:
        temp = (episode_interval, eps)
        episode_templates.append(temp)

# Generate the episode callbacks
episode_callbacks = get_episode_callbacks(episode_templates, training_agent, agent_configuration.warm_up_steps,
                                          epochs, save_epoch_interval, log_interval)


##############
# Experiment #
##############
experiment = Experiment(experiment_configuration, agent_configuration, model_configuration,
                        tree_configurations, episode_callbacks, sdt_threads=8)
env_description = get_env_description(experiment_configuration.env_name)


if __name__ == '__main__':
    gym.logger.set_level(gym.logger.DEBUG)
    torch.set_num_threads(1)  # Any other value than single core creates overhead

    # Train DQN agent
    experiment.train(train_agent_only=train_agent_only, verbose=1)
    # Plot DQN results
    dqn_visualizer = DQNVisualizer()
    dqn_visualizer.plot_all(experiment_configuration, training_dqn=True)

    # Test and plot best save
    experiment.test_agent(episodes, log_results=True, visualize=False, use_best=True)
    dqn_visualizer.plot_all(experiment_configuration, training_dqn=False, use_best=True)
    # Test the final agent
    experiment.test_agent(episodes, log_results=True, visualize=False)
    dqn_visualizer.plot_all(experiment_configuration, training_dqn=False)

    # Plot SDT results
    # sdt_visualizer = SDTVisualizer()
    # sdt_visualizer.plot_all(experiment_configuration, tree_configurations, episode_callbacks, save_epoch_interval,
    #                         training_agent)
    # Plot SDT Trees
    # tree_visualizer = TreeVisualizer(env_description)
    # tree_visualizer.visualize_all(experiment_configuration, tree_configurations, episode_callbacks)

    # Compare Policies
    # experiment.test_trees_with_agent(steps=10000, agent_step_save=None, deterministic=True)
    # Plot policy comparisons
    # policy_visualizer = PolicyVisualizer()
    # policy_visualizer.plot_all(experiment_configuration, tree_configurations, episode_callbacks, training_agent)

    # policy_visualizer = PolicyVisualizer()
    # policy_visualizer.plot_avg_comparisons(experiment_configuration, tree_configurations, episode_callbacks,
    #                                        training_agent, range(4))
