from core.experiment.environment_description import get_env_description
from core.experiment.experiment import Experiment

from experiments.lunar_lander import *
from experiments.templates import get_episode_callbacks
from visualization.dqn.dqn_visualization import DQNVisualizer


###########################
# ExperimentConfiguration #
###########################
experiment_configuration = ExperimentConfiguration(
    env_name=env_name,
    seed=0,
    steps=steps,
    no_cuda=no_cuda,
    save_directory=f"../../../../Data/Experiments/LunarLander/epsilon_greedy/post/Prioritised_Agent/"
)
# Whether or not SDTs are trained together with the DQN
train_agent_only = True
# Whether or not the DQN agent is training at the same time as the SDT
training_agent = False


######################
# AgentConfiguration #
######################
# Set the experiment specific policies
agent_configuration.prioritised_replay = True
model_configuration.learning_rate /= 4


#####################
# Episode Callbacks #
#####################
# Generate the episode callbacks
episode_callbacks = get_episode_callbacks(episode_templates, training_agent, agent_configuration.warm_up_steps,
                                          epochs, save_epoch_interval, log_interval)

##############
# Experiment #
##############
# experiment = Experiment(experiment_configuration, agent_configuration, model_configuration,
#                         tree_configurations, episode_callbacks, sdt_threads=8)
env_description = get_env_description(experiment_configuration.env_name)


if __name__ == '__main__':
    gym.logger.set_level(gym.logger.DEBUG)
    torch.set_num_threads(1)  # Any other value than single core creates overhead

    start = 0
    total_seeds = 30
    for s in range(start, start + total_seeds):
        print(f"Starting seed {s}...")
        experiment_configuration.seed = s
        experiment_configuration.save_directory = Path(experiment_configuration.base_save_directory) / f"{s}"
        experiment = Experiment(experiment_configuration, agent_configuration, model_configuration,
                                tree_configurations, episode_callbacks, sdt_threads=8)

        # Train DQN agent
        experiment.train(train_agent_only=train_agent_only, verbose=1)
        # Plot DQN results
        dqn_visualizer = DQNVisualizer()
        dqn_visualizer.plot_all(experiment_configuration, training_dqn=True, colour_idx=1)

        # Test and plot best save
        experiment.test_agent(episodes, log_results=True, visualize=False, use_best=True)
        dqn_visualizer.plot_all(experiment_configuration, training_dqn=False, use_best=True)
        # Test the final agent
        experiment.test_agent(episodes, log_results=True, visualize=False)
        dqn_visualizer.plot_all(experiment_configuration, training_dqn=False)

    # Compare all agents
    # dqn_visualizer = DQNVisualizer()
    # dqn_visualizer.compare_same_config(experiment_configuration, seeds=range(30))
    # dqn_visualizer.plot_seeds_all(experiment_configuration, seeds=range(30), y_lim=(-600, 305))
