from core.experiment.environment_description import get_env_description
from core.experiment.experiment import Experiment

from experiments.mario import *
from experiments.templates import get_episode_callbacks
from lib.mario_gym.gym_envs.mario import UnaryMarioAction
from visualization.dqn.dqn_visualization import DQNVisualizer


###########################
# ExperimentConfiguration #
###########################
vis = 'off'
experiment_configuration = ExperimentConfiguration(
    env_name=env_name,
    seed=0,
    steps=steps,
    no_cuda=no_cuda,
    save_directory=f"../../../../Data/Experiments/Mario/epsilon_greedy/concurrent/Prioritised_SDT_Replay/",
    make_environment=lambda en: UnaryMarioAction(gym.make(en, fps=100, vis=vis, port=7936, time_per_level=300))
)
# Whether or not SDTs are trained together with the DQN
train_agent_only = False
# Whether or not the DQN agent is training at the same time as the SDT
training_agent = True


######################
# AgentConfiguration #
######################
agent_configuration.sdt_replay = True
agent_configuration.random_sampling = False
agent_configuration.prioritised_sampling = True
model_configuration.learning_rate /= 4


#####################
# Episode Callbacks #
#####################
# The collections of depths to generate configurations for
depths = [5]
# The list of tree configurations
tree_configurations = get_tree_configurations(depths, lmbda, sdt_batch_size)

episode_templates = [
    # (episode_interval, episodes)
    (100, 10)
]
# Generate the episode callbacks
episode_callbacks = get_episode_callbacks(episode_templates, training_agent, agent_configuration.warm_up_steps,
                                          epochs, save_epoch_interval, log_interval)

##############
# Experiment #
##############
experiment = Experiment(experiment_configuration, agent_configuration, model_configuration,
                        tree_configurations, episode_callbacks, sdt_threads=8)
env_description = get_env_description(experiment_configuration.env_name)


if __name__ == '__main__':
    gym.logger.set_level(gym.logger.DEBUG)
    torch.set_num_threads(1)  # Any other value than single core creates overhead

    # Train DQN agent
    experiment.train(train_agent_only=train_agent_only, verbose=1)
    # Plot DQN results
    dqn_visualizer = DQNVisualizer()
    num_leafs = 2 ** depths[0]
    dqn_visualizer.plot_all(experiment_configuration, training_dqn=True, num_leafs=num_leafs, colour_idx=3)
    # Test and plot best save
    experiment.test_agent(episodes, log_results=True, visualize=False, use_best=True)
    dqn_visualizer.plot_all(experiment_configuration, training_dqn=False, use_best=True, num_leafs=num_leafs,
                            colour_idx=3)

    # Test the final agent
    experiment.test_agent(episodes, log_results=True, visualize=False)
    dqn_visualizer.plot_all(experiment_configuration, training_dqn=False, num_leafs=num_leafs, colour_idx=3)

    dqn_visualizer = DQNVisualizer()
    dqn_visualizer.compare_same_config(experiment_configuration, seeds=range(1))
    dqn_visualizer.plot_seeds_all(experiment_configuration, seeds=range(1), y_lim=(-855, 155))
