import random
import numpy as np
from lib.per_master.SumTree import SumTree


class Memory:  # stored as ( s, a, r, s_ ) in SumTree
    # Explaining_DRL: moved attributes into init call
    # e = 0.01
    # a = 0.6
    # beta = 0.4
    # beta_increment_per_sampling = 0.001

    def __init__(self, capacity, e=0.01, a=0.6, beta=0.4, beta_increment_per_sampling=0.001):
        self.tree = SumTree(capacity)
        self.capacity = capacity
        self.e = e
        self.a = a
        self.beta = beta
        self.beta_increment_per_sampling = beta_increment_per_sampling

    def _get_priority(self, error):
        return (np.abs(error) + self.e) ** self.a

    def add(self, error, sample):
        p = self._get_priority(error)
        self.tree.add(p, sample)

    def sample(self, n):
        batch = []
        idxs = []
        segment = self.tree.total() / n
        priorities = []

        self.beta = np.min([1., self.beta + self.beta_increment_per_sampling])

        for i in range(n):
            a = segment * i
            b = segment * (i + 1)

            # Explaining_DRL: avoid sampling a 0 integer (uninitialised data) => re-sample segment
            # Based on issue https://github.com/rlcode/per/issues/4
            # (Should not happen if warm-up steps fill entire memory)
            while True:
                s = random.uniform(a, b)
                (idx, p, data) = self.tree.get(s)
                if not isinstance(data, int):
                    break
                else:
                    print("Prioritised replay, sampled uninitialized tree node...", i, a, b, s)
            #
            priorities.append(p)
            batch.append(data)
            idxs.append(idx)

        sampling_probabilities = priorities / self.tree.total()
        is_weight = np.power(self.tree.n_entries * sampling_probabilities, -self.beta)
        is_weight /= is_weight.max()

        return batch, idxs, is_weight

    def update(self, idx, error):
        p = self._get_priority(error)
        self.tree.update(idx, p)
