import math
from typing import List

import numpy as np

from core.callbacks.sdt.episode_callback import EpisodeCallbackTemplate
from core.callbacks.sdt.sdt_callback import SDTCallbackTemplate
from core.experiment.config import ExperimentConfiguration


def print_loading(path):
    print("Loading", path, end="...")


def print_num_entries(nb_entries):
    print("No data left." if nb_entries == 0 else f"{nb_entries} entries found.")


def string_to_list(string, d_type=np.float32):
    new_string = string[1:len(string)-1].split(" ")
    # Make sure there are no empty strings in the list
    new_list = [s for s in new_string if len(s) > 0]
    return np.array(new_list, dtype=d_type)


def compare_per_tree_config(sdt_callbacks: List[SDTCallbackTemplate], dqn_loader,
                            exp_config: ExperimentConfiguration, training_dqn):
    """Sort the comparisons to make the plots more readable."""
    total_episodes = dqn_loader.get_last_episode(exp_config, training_dqn, use_best=False)
    print(f"Found DQN agent played {total_episodes} episodes while SDT were training.")
    per_interval = {}
    per_episodes = {}
    per_full_int = []
    per_episodes_used = {}
    for callback in sdt_callbacks:
        if isinstance(callback, EpisodeCallbackTemplate):
            episode_interval = callback.episode_interval
            episodes = callback.episodes
            episodes_used = math.floor(total_episodes / episode_interval * episodes)
            # Compare per interval
            if per_interval.get(episode_interval) is None:
                per_interval[episode_interval] = []
            per_interval[episode_interval].append(callback)
            # Compare per episodes used
            if per_episodes.get(episodes) is None:
                per_episodes[episodes] = []
            per_episodes[episodes].append(callback)
            # Compare per full interval used
            if episode_interval == episodes:
                per_full_int.append(callback)
            # Only add episodes used for non-full intervals
            if episodes != episode_interval:
                # Compare per episode used
                if per_episodes_used.get(episodes_used) is None:
                    per_episodes_used[episodes_used] = []
                per_episodes_used[episodes_used].append(callback)

    # Return the comparison matches
    return per_interval, per_episodes, per_full_int, per_episodes_used


def compare_pre_trained_concurrent(pre_trained_callbacks: List[SDTCallbackTemplate],
                                   concurrent_callbacks: List[SDTCallbackTemplate]):
    """Sort the comparisons to make the plots more readable."""
    matches = []
    # Iterate over the lists
    for pre_trained in pre_trained_callbacks:
        if isinstance(pre_trained, EpisodeCallbackTemplate):
            for concurrent in concurrent_callbacks:
                if isinstance(concurrent, EpisodeCallbackTemplate):
                    # Only compare the callbacks if they have the same interval and episodes used
                    if pre_trained.episode_interval == concurrent.episode_interval and \
                            pre_trained.episodes == concurrent.episodes:
                        matches.append((pre_trained, concurrent))
    # Return the matches
    return matches
