import csv

from core.callbacks.loggers.action_logger import PostActionsLog
from core.callbacks.sdt.sdt_callback import SDTCallbackTemplate
from core.experiment.config import ExperimentConfiguration, TreeConfiguration
from core.experiment.environment_description import get_env_description
from core.paths.compare_path import ComparePaths
from visualization import print_loading, print_num_entries


class PolicyLoader(object):
    """The loader for csv for SDT actions."""

    @staticmethod
    def load_actions_data(exp_config: ExperimentConfiguration,
                          tree_config: TreeConfiguration, sdt_callback: SDTCallbackTemplate):
        env_description = get_env_description(exp_config.env_name)
        actions_log = PostActionsLog(env_description, tree_config)
        # Define the data to be loaded
        data = {field: [] for field in actions_log.entry_fields}

        # Load the data
        path = ComparePaths.log_specific_path(exp_config, tree_config, sdt_callback.create_name())
        print_loading(path)
        with open(path, "r") as file:
            reader = csv.reader(file)
            nb_entries = 0
            fields = None
            for entry in reader:
                # Read the header
                if fields is None:
                    fields = entry
                # Read the entries
                else:
                    # Load the data
                    for field, value in zip(fields, entry):
                        # Value may be an empty string (for unused nodes)
                        if len(value) == 0:
                            value = 0
                        # Action is a name
                        elif field == actions_log.action:
                            pass
                        # Accuracy and average are floats
                        elif field == actions_log.accuracy or field == actions_log.order_avg:
                            value = float(value)
                        else:
                            value = int(value)
                        # Store the entry value
                        data[field].append(value)
                    # An other entry has been loaded
                    nb_entries += 1
            print_num_entries(nb_entries)
        # Return the data
        return data
