import matplotlib.pyplot as plt

from core.experiment.config import TreeConfiguration
from visualization.theme import *


class LeafPlotter(object):
    """The visualizer for a soft decision tree."""

    def __init__(self, tree_configuration: TreeConfiguration,
                 theme: ColourTheme = BlueColours(), lines: LineStyles = LineStyles(), dpi=300):
        self.tree_config = tree_configuration
        self.colours = theme
        self.lines = lines
        self.dpi = dpi

        self.data = {}

    @staticmethod
    def _init_plot(title, x_label, y_label):
        plt.title(title)
        plt.ylabel(y_label)
        plt.xlabel(x_label)

    def _finish_plot(self, save_path, dpi, show_plot, show_labels):
        if show_labels:
            plt.legend()
        plt.savefig(save_path, dpi=dpi)
        if show_plot:
            plt.show()
            plt.close()
        else:
            # plt.show()
            plt.close()
        self.colours.reset()
        self.lines.reset()

    def add_entry(self, episode, rgb_array, action, chosen_leaf, leaf_distribution):
        entry = (rgb_array, action, chosen_leaf.item(), leaf_distribution)
        ep_data = self.data.get(episode)
        if ep_data is None:
            self.data[episode] = [entry]
        else:
            self.data[episode].append(entry)

    def plot_chosen_leafs(self, save_path, episodes=None, show_plot=False):
        episodes = episodes if episodes is not None else range(len(self.data))

        for ep in episodes:
            ep_data = self.data[ep]
            steps = range(len(ep_data))
            actions = []
            chosen_leafs = []
            # leaf_distributions = []

            for (rgb_array, action, chosen_leaf, leaf_distribution) in ep_data:
                actions.append(action)
                chosen_leafs.append(chosen_leaf)
                # leaf_distributions.append(leaf_distribution)

            # Plot chosen leaf per step
            file_path = f"{save_path}/Episode_{ep}_chosen_leafs.png"
            self._init_plot(f"Episode {ep} | Chosen leaf per step", x_label="Step", y_label="Chosen Leaf")
            plt.scatter(steps, chosen_leafs, cmap=self.colours, lw=0.1)
            self._finish_plot(file_path, self.dpi, show_plot, show_labels=False)

            # Plot chosen action per step
            file_path = f"{save_path}/Episode_{ep}_actions.png"
            self._init_plot(f"Episode {ep} | Chosen action per step", x_label="Step", y_label="Chosen Action")
            plt.scatter(steps, actions, cmap=self.colours, lw=0.1)
            self._finish_plot(file_path, self.dpi, show_plot, show_labels=False)
